#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include "perplex.h"

#define PA2BAR (1.0/100000.0)
#define M2KM (1.0/1000.0)
#define ADIABAT 0.35
#define MANTLETEMP 1623.0
#define DENSITY 3200.0
#define G 9.81

#define POINTS 400
#define DMIN 5.0
#define DMAX 150.0

#define NCOMP 7

#define MELTPHASE "melto"
#define MELTPHASELEN 5 

#define SOLUTION_MODEL "pierre"

double geoP(double d) {
    return DENSITY * G * d * PA2BAR;
}

double geoT(double d) {
    return MANTLETEMP + ADIABAT * d * M2KM;
}

int main(int argc, char **argv) {
    double dmin = DMIN / M2KM;
    double dmax = DMAX / M2KM;
    int points = POINTS;
    int ncomp = NCOMP;
    
    double d, dd, P, T, sum, totfrac;
    
    double *comp, *histcomp, *tmpcomp;
    FILE *fp;

	int nphases, retval;
	double *wtphases = 0;
	double *cphases = 0;
	char *namephases = 0;
    double *sysprop = 0;
    

    dd = (dmax - dmin) / (double)points;

    comp = (double *)calloc(ncomp, sizeof(double));
    tmpcomp = (double *)calloc(ncomp, sizeof(double));
    histcomp = (double *)calloc(ncomp * points, sizeof(double));
    
    if (comp == NULL || histcomp == NULL || tmpcomp == NULL) {
        fprintf(stderr, "!! Error.\n");
        exit(1);
    }

	fp = fopen("output.csv", "w");
	if (fp == 0) {
		fprintf(stderr, "!! Failed to open output.csv\n");
		exit(1);
	}
    
    // not the order!
    comp[6] = 0.3911; // mgo
    comp[1] = 0.0402; // al2o3
    comp[2] = 0.4514; // sio2
    comp[3] = 0.0320; // cao
    comp[4] = 0.0826; // feo
    comp[5] = 0.0031; // na2o
    comp[0] = 0.0000; // h2o

    ini_phaseq(SOLUTION_MODEL);

    print_comp_order();
    
    totfrac = 1.0;
    
    fprintf(fp, "i,P,T,totfrac,meltwtperc");
    for (int j = 0; j < ncomp; j++) {
        fprintf(fp, ",meltcomp%d",j);
    }
    for (int j = 0; j < ncomp; j++) {
        fprintf(fp, ",remainingcomp%d",j);
    }
    fprintf(fp, ",phasespresent...\n");
        
    for (int i = 0; i < points; i++) {
        int meltfound;
        
        d = dmax - i * dd;
        P = geoP(d);
        T = geoT(d);
        
        retval = phaseq(P, T, ncomp, comp, &nphases, &wtphases, &cphases, &sysprop, &namephases, 0);
        if (retval != 0) {
            fprintf(stderr, "!! PerpleX failed. Exiting.\n");
            exit(retval);
        }
        
        meltfound = -1;
        for (int j = 0; j < nphases; j++) {
            if (strncmp(namephases + j*p_pname_len, MELTPHASE, MELTPHASELEN) == 0) {
                meltfound = j;
//                fprintf(stderr, "%.14s, %g\n", namephases + j*p_pname_len, wtphases[meltfound] / 100.0);
                break;
            }
        }

        if (meltfound >= 0) {
            wtphases[meltfound] = wtphases[meltfound] / 100.;
            totfrac = totfrac - wtphases[meltfound] * totfrac;
            sum = 0;
            for (int j = 0; j < ncomp; j++) {
                cphases[meltfound * ncomp + j] /= 100.;
                tmpcomp[j] = (comp[j] - cphases[meltfound * ncomp + j] * wtphases[meltfound]) / (1. - wtphases[meltfound]);
                sum += tmpcomp[j];
            }
        }

        if (meltfound >= 0) {
            fprintf(fp, "%d,%g,%g,%g,%g", i, P, T, totfrac, wtphases[meltfound]);
        } else {
            fprintf(fp, "%d,%g,%g,%g,%g", i, P, T, totfrac, 0.0);
        }

        for (int j = 0; j < ncomp; j++) {
            // melt comp
            if (meltfound >= 0) {
                fprintf(fp, ",%g", cphases[meltfound * ncomp + j]);
            } else {
                fprintf(fp, ",NA");
            }
        }

        for (int j = 0; j < ncomp; j++) {
            // remaining comp
            if (meltfound >= 0) {
                comp[j] = tmpcomp[j] / sum;
            }
            fprintf(fp, ",%g", comp[j]);
        }

        fprintf(fp, ",");
        for (int j = 0; j < nphases; j++) {
            fprintf(fp, " %.14s", namephases + j*p_pname_len);
        }
        fprintf(fp, "\n");
    }
    
    fclose(fp);
    	
	return 0;
}

