#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "perplex.h"

#define PA2BAR (1.0/100000.0)
#define M2KM (1.0/1000.0)
#define ADIABAT 0.35
#define MANTLETEMP 1673.0
#define DENSITY 3200.0
#define G 9.81

#define POINTS 400
#define DMIN 0.1
#define DMAX 150.0

#define NCOMP 7

#define MELTPHASE "melto"
#define MELTPHASELEN 5 

#define MELTLIMIT 0.15  // only remove melt when it exceeds 0.1 %

#define SOLUTION_MODEL "pierre"
#define PRINT_INTERMEDIATE_MELT 1


// TODO: - calc molar Mg#  DONE
//       - print intermediate melt compositions (those not being extracted)   DONE
//       - check handling of the solvi

double geotherm(int type, double z);

double geoP(double d) {
    return DENSITY * G * d * PA2BAR;
}

double geoT(double d) {
    return MANTLETEMP + ADIABAT * d * M2KM;
}

int main(int argc, char **argv) {
    double dmin = DMIN / M2KM;
    double dmax = DMAX / M2KM;
    int points = POINTS;
    int ncomp = NCOMP;
    
    double d, dd, P, T, sum, totfrac;
    
    double *comp, *histcomp, *tmpcomp;
    FILE *fp;

	int nphases, retval;
	double *wtphases = 0;
	double *cphases = 0;
	char *namephases = 0;
    double *sysprop = 0;

    const int c_out_name_len = 6;
    const char *c_out_name[c_out_name_len+1]; // +1 for \0
    int i, extract;
    int mgo_i, feo_i;
    

    dd = (dmax - dmin) / (double)points;

    comp = (double *)calloc(ncomp, sizeof(double));
    tmpcomp = (double *)calloc(ncomp, sizeof(double));
    histcomp = (double *)calloc(ncomp * points, sizeof(double));
    
    if (comp == NULL || histcomp == NULL || tmpcomp == NULL) {
        fprintf(stderr, "!! Error.\n");
        exit(1);
    }

	fp = fopen("output.csv", "w");
	if (fp == 0) {
		fprintf(stderr, "!! Failed to open output.csv\n");
		exit(1);
	}
    
    // not the order!
    i = 6; comp[i] = 0.3911; c_out_name[i] = "mgo"; mgo_i = i;
    i = 1; comp[i] = 0.0402; c_out_name[i] = "al2o3"; 
    i = 2; comp[i] = 0.4514; c_out_name[i] = "sio2"; 
    i = 3; comp[i] = 0.0320; c_out_name[i] = "cao"; 
    i = 4; comp[i] = 0.0826; c_out_name[i] = "feo"; feo_i = i;
    i = 5; comp[i] = 0.0031; c_out_name[i] = "na2o"; 
    i = 0; comp[i] = 0.0000; c_out_name[i] = "h2o"; 


    ini_phaseq(SOLUTION_MODEL);

    print_comp_order();
    
    totfrac = 1.0;
    
    fprintf(fp, "i,P,T,totfrac,meltwtperc");
    for (int j = 0; j < ncomp; j++) {
        fprintf(fp, ",melt_%s",c_out_name[j]);
    }
    fprintf(fp, ",mgnum");
    for (int j = 0; j < ncomp; j++) {
        fprintf(fp, ",solid_%s",c_out_name[j]);
    }
    fprintf(fp, ",phasespresent\n");
        
    for (int i = 0; i < points; i++) {
        int meltfound;
        
        d = dmax - i * dd;
        P = geoP(d);
        T = geoT(d);  // mantle potential temperature + adiabat
        //T = geotherm(10, d*M2KM); // lithosphere geotherm + mantle geotherm
        
        retval = phaseq(P, T, ncomp, comp, &nphases, &wtphases, &cphases, &sysprop, &namephases, 0);
        if (retval != 0) {
            fprintf(stderr, "!! PerpleX failed. Exiting.\n");
            exit(retval);
        }
        
        meltfound = -1;
        for (int j = 0; j < nphases; j++) {
            if (strncmp(namephases + j*p_pname_len, MELTPHASE, MELTPHASELEN) == 0) {
                meltfound = j;
//                fprintf(stderr, "%.14s, %g\n", namephases + j*p_pname_len, wtphases[meltfound] / 100.0);
                break;
            }
        }

        extract = 0;
        if (meltfound >= 0) {
            wtphases[meltfound] = wtphases[meltfound] / 100.;
            fprintf(stderr, " Melt: %f\n", wtphases[meltfound]);
            if (wtphases[meltfound] > MELTLIMIT)  {
                extract = 1;
                totfrac = totfrac - wtphases[meltfound] * totfrac;
            } else {
                fprintf(stderr, " Discarding melt <%f%%\n", MELTLIMIT);
            }
            sum = 0;
            for (int j = 0; j < ncomp; j++) {
                cphases[meltfound * ncomp + j] /= 100.;
                if (extract) {
                    tmpcomp[j] = (comp[j] - cphases[meltfound * ncomp + j] * wtphases[meltfound]) / (1. - wtphases[meltfound]);
                    sum += tmpcomp[j];
                }
            }
        }

        if (meltfound >= 0 && extract) {
            fprintf(fp, "%d,%g,%g,%g,%g", i, P, T, totfrac, wtphases[meltfound]);
        } else {
            fprintf(fp, "%d,%g,%g,%g,%g", i, P, T, totfrac, wtphases[meltfound]);
        }

        for (int j = 0; j < ncomp; j++) {
            // melt comp
            if (meltfound >= 0) {
                fprintf(fp, ",%g", cphases[meltfound * ncomp + j]);
            } else {
                fprintf(fp, ",NA");
            }
        }

        // molar mg number
        if (meltfound >= 0) {
            fprintf(fp, ",%g", (cphases[meltfound * ncomp + mgo_i] / 40.305) / (cphases[meltfound * ncomp + mgo_i]/40.305 + cphases[meltfound * ncomp + feo_i]/71.845));
        } else {
            fprintf(fp, ",NA");
        }

        for (int j = 0; j < ncomp; j++) {
            // remaining comp
            if (meltfound >= 0 && extract) {
                comp[j] = tmpcomp[j] / sum;
            }
            fprintf(fp, ",%g", comp[j]);
        }

        fprintf(fp, ",");
        for (int j = 0; j < nphases; j++) {
            fprintf(fp, " %.14s", namephases + j*p_pname_len);
        }
        fprintf(fp, "\n");
    }
    
    fclose(fp);
    	
	return 0;
}



double geotherm(int type, double z) {
    /* return geotherm, copy from citcom */

    /* z: depth in km */
    double temp; /* output temperature, in deg C */

    if (type == 10 || type == 11) {
        double ztmp;
        double c0, c1, c2, c3, c4, c5, c6, c7, c8, c9;
        ztmp = z;
        z = z / 660.0;
        if (type == 10) {
            c0 =  1.651125e-03;
            c1 =  4.629000e+00;
            c2 =  1.047416e+02;
            c3 = -1.198880e+03;
            c4 =  5.502716e+03;
            c5 = -1.382073e+04;
            c6 =  2.050299e+04;
            c7 = -1.792875e+04;
            c8 =  8.548813e+03;
            c9 = -1.714484e+03;
        } else if (type == 11) {
            /* like 10 but input file had visc0=0.5, melting off */
            c0 = -2.485832e-03;
            c1 =  5.994699e+00;  
            c2 =  9.710180e+01; 
            c3 = -1.247227e+03;  
            c4 =  6.051612e+03; 
            c5 = -1.584579e+04;  
            c6 =  2.434523e+04; 
            c7 = -2.195764e+04;  
            c8 =  1.076886e+04; 
            c9 = -2.217088e+03;
        }
        temp = 
            c0 + 
            c1 * pow(z, 1.0) + 
            c2 * pow(z, 2.0) + 
            c3 * pow(z, 3.0) + 
            c4 * pow(z, 4.0) +
            c5 * pow(z, 5.0) + 
            c6 * pow(z, 6.0) + 
            c7 * pow(z, 7.0) + 
            c8 * pow(z, 8.0) +
            c9 * pow(z, 9.0);
        if (temp < 0 || temp > 2.0) { fprintf(stderr, "temp: %f\n", temp); temp=0.0; /*exit(1);*/ }
        temp = temp * 1350.0 + 273.0 + ADIABAT * ztmp * M2KM;
        z = ztmp;
    } else {
        temp = 0;
    }

    return temp;

}
