#include <stdio.h>
#include <malloc.h>
#include "perplex.h"

/* This example will use PerpleX wrapper to evaluate
   the stable phases of given compositions (array 'comp'
   below) in pressure-temperature range from 1 bar/1373 K
   to 35 kbar/2173 K, and output them to output.csv 
*/

int main(int argc, char **argv) {
	double P, T;
	int ncomp;
	double *comp;
	int i, j, row;

	FILE *fp;

	double p1, p2, t1, t2;
	double dp, dt;
	
	int nphases;
	double *wtphases = 0;
	double *cphases = 0;
	char *namephases = 0;
	
	/* only call once per build file */
	ini_phaseq("in26klb_new");	// filename (without .dat extension) 
	                                // generated by PerpleX BUILD.
					// If no path defined, it will be looked
					// for in the current folder where
					// this program is executed.
	
	p1 = 35000;
	p2 = 35000;
	t1 = 1173;
	t2 = 1173;

	dp = 250;
	dt = 20;	

	ncomp = 6;

	fp = fopen("output.csv", "w");
	if (fp == 0) {
		fprintf(stderr, "!! Failed to open output.csv\n");
		return -1;
	}
	comp = (double*)calloc(ncomp, sizeof(double));

	fprintf(fp, "\"P\",\"T\",\"phase\",\"phasewt\"");
	for (j = 0; j < ncomp; j++) {
		fprintf(fp, ",\"comp%d\"", j);
	}
	fprintf(fp, "\n");

	row = 0;

	for (P = p1; P <= p2; P += dp) {
		for (T = t1; T <= t2; T += dt) {
			fprintf(stderr, "\rP = %g   T = %g        ", P, T);
			/* MM3 */
                        //comp[0] = 0.4587;
                        //comp[1] = 0.0404;
                        //comp[2] = 0.0728;
                        //comp[3] = 0.3887;
                        //comp[4] = 0.0362;
                        //comp[5] = 0.0031;

			/* MPY-87 */
			//comp[0] = 0.4432;
			//comp[1] = 0.0433;
			//comp[2] = 0.0982;
			//comp[3] = 0.3684;
			//comp[4] = 0.0334;
			//comp[5] = 0.0039;

                        /* KLB-1 */
			/* Array comp should have the composition in the same order
			   that is used in the configuration file generated by BUILD. */
                        comp[0] = 0.4448;
                        comp[1] = 0.0359;
                        comp[2] = 0.0810;
                        comp[3] = 0.3922;
                        comp[4] = 0.0344;
                        comp[5] = 0.0030;
	
			phaseq(P, T, ncomp, comp, &nphases, &wtphases, &cphases, &namephases, 1);
	
			for (i = 0; i < nphases; i++) {
				row++;
				fprintf(fp, "%g,%g,", P, T);			// P , T ,
				fprintf(fp, "%.14s,", namephases + i*p_pname_len);	// name ,
				fprintf(fp, "%.5e", wtphases[i]);			// wt% ,
				for (j = 0; j < ncomp; j++) {
					fprintf(fp, ",%.5e", cphases[i*ncomp + j]);	// composition
				}
				fprintf(fp, "\n");
			}
		}
	}

	free(comp);
	fclose(fp);

	fprintf(stderr, "\n\n");
	
	return 0;
}

